local frame = CreateFrame("Frame")
frame:RegisterUnitEvent("UNIT_AURA", "player")
frame:RegisterEvent("ADDON_LOADED")
frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

print("<WarriorTankAnnounce> by LeoTheMighty @ Crucible (Thunderfury)")
 
local function newHandler(inSpellId, inExecution)
  local handler = {
    auraPresence = 0,
    spellId = inSpellId,
    execution = inExecution
  }
  return handler
end

local function announce(msg)

  local WarningUsed = 0
  
  if RaidWarningVar and IsInRaid() and UnitIsGroupLeader("player") then 
	  SendChatMessage(msg, "RAID_WARNING")
    WarningUsed = 1
	end

  if PartyRaidVar then 
    if IsInRaid() and WarningUsed == 0 then 
      SendChatMessage(msg, "RAID")
    end
    if not IsInRaid() and IsInGroup() then 
      SendChatMessage(msg, "PARTY")
    end
  end

  if YellVar == true and IsInInstance() then
	  SendChatMessage(msg, "YELL")
	end

  if SayVar == true and IsInInstance() then
	  SendChatMessage(msg, "SAY")
	end

end

local function handleBuffEvent(myTab)
  local found = 0
  for i = 1, 40 do
    local _, _, _, _, _, _, _, _, _, spellId = UnitAura("player", i)
    
    -- if spellId then print("Buff " .. i .. " = " .. spellId) end
    
    if spellId == myTab.spellId then
      found = 1
	  break
    end
  end
  if found == 1 then
    if myTab.auraPresence == 0 then
      myTab.auraPresence = 1
      local playerName = UnitName("player");
      myTab.execution(playerName, myTab)
    end
  else
    myTab.auraPresence = 0
  end
end

local function handleDebuffEvent(myTab)
  local found = 0
  for i = 1, 40 do
    local _, _, _, _, _, _, _, _, _, spellId = UnitAura("player", i, "HARMFUL")
    
    -- if spellId then print("Debuff " .. i .. " = " .. spellId) end

    if spellId == myTab.spellId then
      found = 1
	  break
    end
  end
  if found == 1 then
    if myTab.auraPresence == 0 then
      myTab.auraPresence = 1
      local playerName = UnitName("player");
      myTab.execution(playerName, myTab)
    end
  else
    myTab.auraPresence = 0
  end
end

local function handleTick(tickState, myTab, timeObj, player)
  if tickState.maxTicks < 0 then
    timeObj:Cancel()
  else
    if myTab.auraPresence == 0 then
      tickState.maxTicks = 0
    end
    if tickState.maxTicks == 0 then
      announce(format("%s ended", tickState.auraName))
    elseif tickState.maxTicks <= tickState.countDown then
      announce(format("%s ends in %ds", tickState.auraName, tickState.maxTicks))
    end
  end
  tickState.maxTicks = tickState.maxTicks-1
end

local function lastStand(player, myTab)
  announce("Last Stand (+30% health, 20 secs)")
  local tickState = {maxTicks = 18, auraName = "Last Stand", countDown = LastStandCountDownVar}
  C_Timer.NewTicker(1, function(timerObj) handleTick(tickState, myTab, timerObj, player) end)
end

local function shieldWallTicks()
    local currentRank = select(5, GetTalentInfo(3, 13));
    if currentRank == 1 then
	  return 11
	elseif currentRank == 2 then
      return 13
    end
    return 8
end

local function shieldWall(player, myTab)
  local maxtime = shieldWallTicks() + 2;
  announce("Shield Wall ( " .. maxtime .. " secs)")
  local tickState = {maxTicks = shieldWallTicks(), auraName = "Shield Wall", countDown = ShieldWallCountDownVar}
  C_Timer.NewTicker(1, function(timerObj) handleTick(tickState, myTab, timerObj, player) end)
end

local function lifegivingGem(player, myTab)
  announce("Gift of Life (+15% health, 20 secs)")
  local tickState = {maxTicks = 18, auraName = "Gift of Life", countDown = LifegivingGemCountDownVar}
  C_Timer.NewTicker(1, function(timerObj) handleTick(tickState, myTab, timerObj, player) end)
end

local function deathWish(player, myTab)
  announce("Death Wish (30 secs)")
  local tickState = {maxTicks = 28, auraName = "Death Wish", countDown = 0}
  C_Timer.NewTicker(1, function(timerObj) handleTick(tickState, myTab, timerObj, player) end)
end

local function recklessness(player, myTab)
  announce("Recklessness (15 secs)")
  local tickState = {maxTicks = 13, auraName = "Recklessness", countDown = 0}
  C_Timer.NewTicker(1, function(timerObj) handleTick(tickState, myTab, timerObj, player) end)
end

local lastStandHandler = newHandler(12976, lastStand)
local shieldWallHandler = newHandler(871, shieldWall)
local lifegivingGemHandler = newHandler(23782, lifegivingGem)
local deathWishHandler = newHandler(12292, deathWish)
local recklessnessHandler = newHandler(1719, recklessness)

local sliderLabelUpdate = function(slider)
  local newValue = slider:GetValue()
  text = format("%d seconds", newValue)
  if (newValue == 0) then
    text = "Off"
  end
  getglobal(slider:GetName() .. 'Text'):SetText(text)
end	

local function initSavedVariables()

	if SayVar == nil then
	  SayVar = false
	end
	if YellVar == nil then
	  YellVar = true
	end  
	if PartyRaidVar == nil then
	  PartyRaidVar = false
	end
 	if RaidWarningVar == nil then
	  RaidWarningVar = false
	end

	if LastStandVar == nil then
	  LastStandVar = true
	end
	if ShieldWallVar == nil then
	  ShieldWallVar = true
	end
	if LifegivingGemVar == nil then
	  LifegivingGemVar = true
	end
	if LastStandCountDownVar == nil then
	  LastStandCountDownVar = 3
	end
	if ShieldWallCountDownVar == nil then
	  ShieldWallCountDownVar = 3
	end
	if LifegivingGemCountDownVar == nil then
	  LifegivingGemCountDownVar = 3
	end
  
  if RecklessnessVar == nil then
	  RecklessnessVar = false
	end
  if DeathWishVar == nil then
	  DeathWishVar = false
	end
  
  if TauntFailedSettings == nil then
	  TauntFailedSettings = {mockingBlow=true, taunt=true, challengingShout=true}
	end
end

local function initUi()
	local optionsFrame = CreateFrame( "Frame", "optionsFrame", UIParent);
	optionsFrame.name = "WarriorTankAnnounce";
	InterfaceOptions_AddCategory(optionsFrame);
	
	initSavedVariables()
	
-- Channel Selection  
 
	local sayButton = CreateFrame("CheckButton", "SayButton", optionsFrame, "UICheckButtonTemplate")
	sayButton:SetPoint("TOPLEFT",15,-50)
	sayButton.text:SetText("Announce in /say (only in instance)")
	sayButton:SetChecked(SayVar)
	sayButton:SetScript("OnClick",function() SayVar=not SayVar end)

	local yellButton = CreateFrame("CheckButton", "YellButton", optionsFrame, "UICheckButtonTemplate")
	yellButton:SetPoint("TOPLEFT",15,-80)
	yellButton.text:SetText("Announce in /yell (only in instance)")
	yellButton:SetChecked(YellVar)
	yellButton:SetScript("OnClick",function() YellVar=not YellVar end)	 
	
	local PartyRaidButton = CreateFrame("CheckButton", "PartyRaidButton", optionsFrame, "UICheckButtonTemplate")
	PartyRaidButton:SetPoint("TOPLEFT",15,-110)
	PartyRaidButton.text:SetText("Announce adaptively in /party or /raid")
	PartyRaidButton:SetChecked(PartyRaidVar)
	PartyRaidButton:SetScript("OnClick",function() PartyRaidVar=not PartyRaidVar end)	 
	
 	local raidWarningButton = CreateFrame("CheckButton", "RaidWarningButton", optionsFrame, "UICheckButtonTemplate")
	raidWarningButton:SetPoint("TOPLEFT",15,-140)
	raidWarningButton.text:SetText("Announce in /raidwarning (override /raid)")
	raidWarningButton:SetChecked(RaidWarningVar)
	raidWarningButton:SetScript("OnClick",function() RaidWarningVar=not RaidWarningVar end)	  
  
-- Defensive Group   
  
	local GroupDown1 = 180
	
	local defensiveCooldownGroupLabel = optionsFrame:CreateFontString("DefensiveCooldownGroupLabel", "OVERLAY", "GameFontNormal")
	defensiveCooldownGroupLabel:SetText("Defensive cooldowns:")
	defensiveCooldownGroupLabel:SetPoint("TOPLEFT", 10, - GroupDown1)
	
	local defensiveCooldownGroup = CreateFrame("Frame", "DefensiveCoolDownsGroup", optionsFrame, "ThinBorderTemplate");
	defensiveCooldownGroup:SetSize(300, 110);
	defensiveCooldownGroup:SetPoint("TOPLEFT", 10, -20 - GroupDown1);
	
	local lastStandButton = CreateFrame("CheckButton", "LastStandButton", optionsFrame, "UICheckButtonTemplate")
	lastStandButton:SetPoint("TOPLEFT",15,-30 - GroupDown1)
	lastStandButton.text:SetText("Last Stand")
	lastStandButton:SetChecked(LastStandVar)
	lastStandButton:SetScript("OnClick",function() LastStandVar=not LastStandVar end)
	
	local countDownSliderLastStand = CreateFrame("Slider", "CountDownSliderLastStand", optionsFrame, "OptionsSliderTemplate")
	countDownSliderLastStand:SetOrientation('HORIZONTAL')
	countDownSliderLastStand:SetPoint("TOPLEFT",150,-38 - GroupDown1)
	countDownSliderLastStand:SetMinMaxValues(0, 4)
	
	countDownSliderLastStand:SetValueStep(1)
	countDownSliderLastStand:SetObeyStepOnDrag(true)
	getglobal(countDownSliderLastStand:GetName() .. 'Low'):SetText(nil)
	getglobal(countDownSliderLastStand:GetName() .. 'High'):SetText(nil)
    sliderLabelUpdate(countDownSliderLastStand)
	countDownSliderLastStand:SetScript("OnValueChanged", function(self, newvalue)
		sliderLabelUpdate(countDownSliderLastStand)
		LastStandCountDownVar = newvalue
    end)
	countDownSliderLastStand:SetValue(LastStandCountDownVar)  
  
	local shieldWallButton = CreateFrame("CheckButton", "ShieldWallButton", optionsFrame, "UICheckButtonTemplate")
	shieldWallButton:SetPoint("TOPLEFT",15,-60 - GroupDown1)
	shieldWallButton.text:SetText("Shield Wall")
	shieldWallButton:SetChecked(ShieldWallVar)
	shieldWallButton:SetScript("OnClick",function() ShieldWallVar=not ShieldWallVar end)
	
	local countDownSliderShieldWall = CreateFrame("Slider", "CountDownSliderShieldWall", optionsFrame, "OptionsSliderTemplate")
	countDownSliderShieldWall:SetOrientation('HORIZONTAL')
	countDownSliderShieldWall:SetPoint("TOPLEFT",150,-68 - GroupDown1)
	countDownSliderShieldWall:SetMinMaxValues(0, 4)
	
	countDownSliderShieldWall:SetValueStep(1)
	countDownSliderShieldWall:SetObeyStepOnDrag(true)
	getglobal(countDownSliderShieldWall:GetName() .. 'Low'):SetText(nil)
	getglobal(countDownSliderShieldWall:GetName() .. 'High'):SetText(nil)
    sliderLabelUpdate(countDownSliderShieldWall)
	countDownSliderShieldWall:SetScript("OnValueChanged", function(self, newvalue)
		sliderLabelUpdate(countDownSliderShieldWall)
		ShieldWallCountDownVar = newvalue
    end)
    countDownSliderShieldWall:SetValue(ShieldWallCountDownVar)  

  local lifegivingGemButton = CreateFrame("CheckButton", "LifegivingGemButton", optionsFrame, "UICheckButtonTemplate")
	lifegivingGemButton:SetPoint("TOPLEFT",15,-90 - GroupDown1)
	lifegivingGemButton.text:SetText("Lifegiving Gem")
	lifegivingGemButton:SetChecked(LifegivingGemVar)
	lifegivingGemButton:SetScript("OnClick",function() LifegivingGemVar=not LifegivingGemVar end)

	local countDownSliderLifegivingGem = CreateFrame("Slider", "CountDownSliderLifegivingGem", optionsFrame, "OptionsSliderTemplate")
	countDownSliderLifegivingGem:SetOrientation('HORIZONTAL')
	countDownSliderLifegivingGem:SetPoint("TOPLEFT",150,-98 - GroupDown1)
	countDownSliderLifegivingGem:SetMinMaxValues(0, 4)
	
	countDownSliderLifegivingGem:SetValueStep(1)
	countDownSliderLifegivingGem:SetObeyStepOnDrag(true)
	getglobal(countDownSliderLifegivingGem:GetName() .. 'Low'):SetText(nil)
	getglobal(countDownSliderLifegivingGem:GetName() .. 'High'):SetText(nil)
    sliderLabelUpdate(countDownSliderLifegivingGem)
	countDownSliderLifegivingGem:SetScript("OnValueChanged", function(self, newvalue)
		sliderLabelUpdate(countDownSliderLifegivingGem)
		LifegivingGemCountDownVar = newvalue
    end)
    countDownSliderLifegivingGem:SetValue(LifegivingGemCountDownVar)
  
-- Taunt Group 
  
 	local GroupDown2 = 320

	local tountGroupLabel = optionsFrame:CreateFontString("TountGroupLabel", "OVERLAY", "GameFontNormal")
	tountGroupLabel:SetText("Taunt Fails:")
	tountGroupLabel:SetPoint("TOPLEFT", 10, - GroupDown2)
	
	local tauntGroup = CreateFrame("Frame", "TountGroup", optionsFrame, "ThinBorderTemplate");
	tauntGroup:SetSize(300, 80);
	tauntGroup:SetPoint("TOPLEFT", 10, -20 - GroupDown2);
  
	local tauntButton = CreateFrame("CheckButton", "TauntButton", optionsFrame, "UICheckButtonTemplate")
	tauntButton:SetPoint("TOPLEFT",15,-30 - GroupDown2)
	tauntButton.text:SetText("Taunt")
	tauntButton:SetChecked(TauntFailedSettings.taunt)
	tauntButton:SetScript("OnClick",function() TauntFailedSettings.taunt=not TauntFailedSettings.taunt end)
	
	local challengingShoutButton = CreateFrame("CheckButton", "ChallengingShoutButton", optionsFrame, "UICheckButtonTemplate")
	challengingShoutButton:SetPoint("TOPLEFT",150,-30 - GroupDown2)
	challengingShoutButton.text:SetText("Challenging Shout")
	challengingShoutButton:SetChecked(TauntFailedSettings.challengingShout)
	challengingShoutButton:SetScript("OnClick",function() TauntFailedSettings.challengingShout=not TauntFailedSettings.challengingShout end)	
	
	local mockingBlowButton = CreateFrame("CheckButton", "MockingBlowButton", optionsFrame, "UICheckButtonTemplate")
	mockingBlowButton:SetPoint("TOPLEFT",15,-60 - GroupDown2)
	mockingBlowButton.text:SetText("Mocking Blow")
	mockingBlowButton:SetChecked(TauntFailedSettings.mockingBlow)
	mockingBlowButton:SetScript("OnClick",function() TauntFailedSettings.mockingBlow=not TauntFailedSettings.mockingBlow end)	
	
-- Offensive Group     
  
 	local GroupDown3 = 430

	local offensiveCooldownGroupLabel = optionsFrame:CreateFontString("OffensiveCooldownGroupLabel", "OVERLAY", "GameFontNormal")
	offensiveCooldownGroupLabel:SetText("Offensive cooldowns:")
	offensiveCooldownGroupLabel:SetPoint("TOPLEFT", 10, - GroupDown3)
	
	local offensiveCooldownGroup = CreateFrame("Frame", "OffensiveCoolDownsGroup", optionsFrame, "ThinBorderTemplate");
	offensiveCooldownGroup:SetSize(300, 50);
	offensiveCooldownGroup:SetPoint("TOPLEFT", 10, -20 - GroupDown3);
  
	local deathWishButton = CreateFrame("CheckButton", "DeathWishButton", optionsFrame, "UICheckButtonTemplate")
	deathWishButton:SetPoint("TOPLEFT",15,-30 - GroupDown3)
	deathWishButton.text:SetText("Death Wish")
	deathWishButton:SetChecked(DeathWishVar)
	deathWishButton:SetScript("OnClick",function() DeathWishVar=not DeathWishVar end)  
  
	local recklessnessButton = CreateFrame("CheckButton", "RecklessnessButton", optionsFrame, "UICheckButtonTemplate")
	recklessnessButton:SetPoint("TOPLEFT",150,-30 - GroupDown3)
	recklessnessButton.text:SetText("Recklessness")
	recklessnessButton:SetChecked(RecklessnessVar)
	recklessnessButton:SetScript("OnClick",function() RecklessnessVar=not RecklessnessVar end)    
  
end

local function printTauntFails(timestamp, subevent, _, sourceGUID, sourceName,_,_,_,destName, ...)
	if subevent ~= "SPELL_MISSED" then
	  return
	end
	local playerName = UnitName("player")
	if sourceName == playerName then
	  local taunt = GetSpellInfo(355)
	  local mockingBlow = GetSpellInfo(694)
	  local challengingShout = GetSpellInfo(1161)
      local spellName, _, missType = select(4, ...)
      if ((spellName == taunt and TauntFailedSettings.taunt) or (spellName == challengingShout and TauntFailedSettings.challengingShout) or (spellName == mockingBlow and TauntFailedSettings.mockingBlow)) then
	    announce(spellName .. " FAILED ON " .. destName .. ": " .. missType)
	  end
	end
end

local eventHandle=function(self, event, arg1, ...)
   if event == "ADDON_LOADED" and arg1 == "WarriorTankAnnounce" then
	 initUi()
   elseif event == "UNIT_AURA" then	
	  if LastStandVar == true then
		handleBuffEvent(lastStandHandler)
	  end
	  if ShieldWallVar == true then
		handleBuffEvent(shieldWallHandler)
	  end
	  if LifegivingGemVar == true then
		handleBuffEvent(lifegivingGemHandler)
	  end
	  if DeathWishVar == true then
		handleBuffEvent(deathWishHandler)
    end
	  if RecklessnessVar == true then
		handleBuffEvent(recklessnessHandler)
	  end
   elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then
    printTauntFails(CombatLogGetCurrentEventInfo())
   end
end

frame:SetScript("OnEvent", eventHandle)